/**************************************************************************/
/* Btrieve Te3chnologies, Inc.                                            */
/* Copyright (c) 1990-1994 Btrieve Technologies, Inc.                     */
/* All Rights Reserved                                                    */
/*                                                                        */
/* DDLSVCS.H                                                              */
/*                                                                        */
/* Structure definitions and function prototypes for using DDLSVCS.DLL    */
/*                                                                        */
/**************************************************************************/


#define PATHLEN  65                /* maximum path length = 64 bytes       */

#define TBLLEN   21
#define FIELDLEN 21                /* maximum field/table name length = 20 */
#define MAXKEYS  24
#define USERLEN  30
#define PSWDLEN  9

#define MEM_ERR               1002
#define DICTIONARY_OPEN       3001
#define NO_FIELDS_ERR         3002
#define ACCESS_DENIED         3003
#define FIELD_DEF_ERR         3004


/* Structure Definitions */



/*----------------------------------------------------------------------------
   FIELDMAP

   This structure describes the properties of a specific field in a
   table.
*/
typedef struct fieldmap
{
   BYTE     szFieldName [FIELDLEN]; /* field name                        */
   USHORT   iType;                  /* data type                         */
   USHORT   iOffset;                /* field offset                      */
   USHORT   iLength;                /* field length                      */
   USHORT   iDecimal;               /* decimal places                    */
   BYTE     bAccessRights;          /* user's access rights on the field */
} FIELDMAP;



/*----------------------------------------------------------------------------
   INDEXMAP

   This structure describes the various indexes associated with a table.
*/
typedef struct indexmap
{
   USHORT   iFieldIndex;            /* index to associated field         */
   USHORT   iIndex;                 /* Btrieve index number              */
   USHORT   iSegment;               /* segment index number              */
   USHORT   iFlags;                 /* index attributes                  */
} INDEXMAP;


/*----------------------------------------------------------------------------
   ADDINDEXMAP

   This structure describes the various indexes to be added to a table.
   The szFieldName field is a NULL terminated string representing the
   associated field name.
*/
typedef struct addindexmap
{
   BYTE   szFieldName[FIELDLEN];  /* Fieldname associated with the ndx */
   BYTE   szIndexName[FIELDLEN];  /* Index name; must be segment 0     */
   USHORT iIndex;                 /* Btrieve index number              */
   USHORT iSegment;               /* segment index number              */
   USHORT iFlags;                 /* index attributes                  */
} ADDINDEXMAP;


/*----------------------------------------------------------------------------
   TABLEPROPERTIES

   This structure defines the properties of a dictionary table.  It is
   used in both BtrGetTableProperties() and BtrCreateTable() functions.

   The structure is filled with all relevant properties of the table
   including lists of index and field descriptions.

   The index and field lists are allocated as global memory objects.
   The application is responsible for freeing the memory blocks.

   On calls to BtrCreateTable(), the application fills the structure with
   all neccessary information to create the table.

   On calls to BtrGetTableProperties(), the information describing the
   table is returned.  The memory needed for the index and field lists
   is allocated by the DLL.

*/
typedef struct tableproperties
{
   BYTE      szTableName [TBLLEN];  /* dictionary TableName               */
   BYTE      szPathName [PATHLEN];  /* actual [path\name]                 */
   USHORT    iRecSize;              /* fixed record length                */
   USHORT    bVariable;             /* variable length records flag       */
   ULONG     dMaxRecSize;           /* fixed + max variable size          */
   USHORT    bWriteAccess;          /* set if user has RW access to table */
   USHORT    iIndexCount;           /* count of items in index list       */
   FIELDMAP *lpFieldList;           /* index list handle                  */
   USHORT    iFieldCount;           /* count of items in field list       */
   INDEXMAP *lpIndexList;           /* field list handle                  */
   PSZ       lpAltSeq;              /* alt. collating sequence handle     */
} TABLEPROPERTIES;



/*----------------------------------------------------------------------------
   TABLELIST

   This structure lists the tables defined in a dictionary.
*/
typedef struct tablelist
{
   BYTE   szName[TBLLEN];
} TABLELIST;


/*----------------------------------------------------------------------------
   DDL_VERSION_INFO

   This structure provides version information for both the DDLSVCS module and
   the currently active Btrieve module(s).  If Btrieve requester is active,
   it is signified by the type 'N', and may also have the local client Btrieve
   active.  In this case, the client Btrieve's version information would be
   in the second group of Btrieve versions.  If only the client Btrieve is
   active, it's version inofrmation is in the first group and it's type is
   given as 'W' for Windows, or 'O' for OS/2.
*/

typedef struct versioninfo
{
   USHORT       iDefMajor;
   USHORT       iDefMinor;
   USHORT       iDefType;
   USHORT       iBMajor1;
   USHORT       iBMinor1;
   USHORT       iBType1;
   USHORT       iBMajor2;
   USHORT       iBMinor2;
   USHORT       iBType2;
} DDL_VERSION_INFO;



/*---------------------------------------------------------------------------
    Open dictionary files FILE.DDF, FIELD.DDF, and INDEX.DDF in the
    given directory.  If security is defined for the dictionary, open
    the security dictionary files USER.DDF and RIGHTS.DDF as well.
    The dictionary handle returned is be used with all subsequent
    operations on the dictionary.

    DDLOpenDataDictionary (szPathName,
                           piDictionaryHandle,
                           szUserName,
                           szPassWord)
*/
extern USHORT APIENTRY DDLOpenDataDictionary (PSZ, PUSHORT, PSZ, PSZ);

/*---------------------------------------------------------------------------
    Close files associated with the dictionary handle

    DDLCloseDataDictionary (iDictionaryHandle)
*/
extern USHORT APIENTRY DDLCloseDataDictionary (USHORT);


/*---------------------------------------------------------------------------
    Create dictionary files FILE.DDF, FIELD.DDF, and INDEX.DDF in the
    given directory.  The handle to the dictionary is returned.  An
    alternate collating can be specified as a resource handle.  The
    resource should be unlocked.  For specifications on an alternate
    collating sequence refer to the Btrieve manual.

    DDLCreateDataDictionary (szPathName,
                             piDictionaryHandle,
                             hAltSeq)
*/
extern USHORT APIENTRY DDLCreateDataDictionary (PSZ, PUSHORT, PSZ);


/*----------------------------------------------------------------------------
    Get the list of tables defined in the specified dictionary

    DDLGetTableNames (iDictionaryHandle, piTableCount, pTableList)
*/
extern USHORT APIENTRY DDLGetTableNames (USHORT, PUSHORT, TABLELIST **);


/*----------------------------------------------------------------------------
    Get information for the specified table in the dictionary

    DDLGetTableProperties (iDictionaryHandle,
                           pTableProps)
*/
extern USHORT APIENTRY DDLGetTableProperties (USHORT, TABLEPROPERTIES *);


/*----------------------------------------------------------------------------
    Create a table in the specified dictionary given the TABLEPROPERTIES

    DDLCreateTable (piDictionaryHandle,
                    pTableProperties)
*/
extern USHORT APIENTRY DDLCreateTable (USHORT, TABLEPROPERTIES *);


/*----------------------------------------------------------------------------
    Drop a table in the specified dictionary

    DDLDropTable (iDictionaryHandle,
                  szTableName)
*/
extern USHORT APIENTRY DDLDropTable (USHORT, PSZ);


/*----------------------------------------------------------------------------
    Add a new index to the existing table specifications for a specified
    table. Multiple index definitions may be added with a single API call.
    The iIndexCount parameter specifies the number of index defintions in the
    list.

    The application should first successfully create a new index in the actual
    data file before this API is called to add the index to the dictionary.

    Index names must be specified in the 0 segment of the index definition.
    If the index is not a named index, set the szIndexName to NULL;

    DDLAddIndex (iDictionaryHandle,
                 szTableName,
                 iIndexCount,
                 pIndexList)
*/
extern USHORT APIENTRY DDLAddIndex (USHORT, PSZ, USHORT, ADDINDEXMAP *);


/*----------------------------------------------------------------------------
    Drop an index in an existing table in the specified dictionary.  The
    reorderIndexes flag applies to Btrieve v6.0 and later.  If the indexes
    are reordered, the flag is returned as TRUE; otherwise, it is FALSE.
    If the applications initializes the flag to FALSE, no reordering will be
    done.

    As indexes may be given a name, the index name may be used rather than
    the index number.  In this case the input index number is ignored.  If the
    index to be dropped is not a named index, pass the szIndexName as NULL.

    DDLDropIndex (iDictionaryHandle,
                  szTableName,
                  szIndexName,
                  iIndex,
                  piReorderIndexes)
*/
extern USHORT APIENTRY DDLDropIndex (USHORT, PSZ, PSZ, USHORT, PUSHORT);


/*----------------------------------------------------------------------------
     Get version information for the DDLSVCS and Btrieve modules.  The Btrieve
     module(s) may be either requester, client, or both.  If both requester
     and client are active, the requester will be the first version in the
     structure and signified by a type of 'N'.

     DDLGetVersionInfo (pVersionInfo)
*/
extern USHORT APIENTRY DDLGetVersionInfo (DDL_VERSION_INFO *);
