Filename:   README.DDL
Product:    Btrieve Data Dictionary Language
Platform:   OS/2
Date:       February 1995

This Readme document describes DDLSVCS.DLL, the Btrieve Data
Dictionary Language (DDL) module for the OS/2 operating system
environment. 

===============================================================================
Disclaimer
==========

Btrieve Technologies, Inc. makes no representations or warranties
with respect to any NetWare software, and specifically disclaims
any express or implied warranties of merchantability, title, or
fitness for a particular purpose.  

Distribution of any Btrieve Technologies software is forbidden
without the express written consent of Btrieve Technologies, Inc.
Further, Btrieve Technologies reserves the right to discontinue
distribution of any Btrieve Technologies software.

Btrieve Technologies is not responsible for lost profits or revenue,
loss of use of the software, loss of data, costs of re-creating lost
data, the cost of any substitute equipment or program, or claims by
any party other than you.  Btrieve Technologies strongly recommends
a backup be made before any software is installed.   Technical
support for this software may be provided at the discretion of
Btrieve Technologies.

===============================================================================
Contents
========

Introduction
API Reference
Main Structures
Status Codes
List of Files
Customer Support


===============================================================================
Introduction
============

While Btrieve provides your application a high performance
key-indexed record management system, it does not provide the
security features, additional data independence, and data integrity
provided by XQL and Scalable SQL (formerly NetWare SQL). Nor does it 
provide XQL's and Scalable SQL's ability to manipulate data with data 
dictionaries.  

A data dictionary is a collection of files that contain information
about Btrieve data files, such as the length and type of data within
each record, the keys in each record, and so on. The data dictionary
files are easily identified by their filename extension (.DDF).

The Btrieve DDL module (DDLSVCS.DLL), when combined with Btrieve, is
an interim programming solution that allows you to write Btrieve
applications that can create and modify data dictionaries without
requiring that either XQL or Scalable SQL be installed. 

NOTE:   While the Btrieve DDL does allow you to write applications 
        that manipulate the data dictionary, it does not provide the 
        other features of XQL and Scalable SQL (security, additional 
        data independence, and data integrity).

The Btrieve DDL module handles all access to the data dictionary,
including security verification and dictionary updates (such
as creating or deleting tables). 

Once your application can access a data dictionary, it can manipulate
the information in the Btrieve data files associated with that
dictionary in much the same way that XQL or Scalable SQL does.  For
example, by using one of the DDL modules, your application can
perform the following tasks:

        * Retrieve the information for any given table in the 
             dictionary (including field and index definitions 
             for that table). 
        * Create or delete tables. (Currently the only constraint 
             when creating new tables is that the DDL's do not 
             allow you to specify field attributes when creating 
             a table. Field attributes must be defined by using
             Scalable SQL APIs or SQL statements.
        * Create a new dictionary. (However, it cannot define 
             security for that dictionary; security can only be 
             defined by using Scalable SQL APIs or SQL statements.
 
        * Retrieve only certain portions of Btrieve records

NOTE:   The functions provided as part of the Btrieve DDL deal only
        with data dictionary files; they do not alter the associated
        Btrieve data file in any way. For this reason, your
        application is responsible for insuring that index
        definitions (flag settings, field offsets, and so on) are 
        consistent with the associated Btrieve data file.

        Make sure that your application unloads Btrieve prior to termination.
        To unload Btrieve for OS/2 the OS/@ application must perform
        Btrieve operation 25 (STOP) using BTRV or BTRCALL function.  Failure
        to do so will result in Btrieve and DDLSVCS remaining in memory
        after application terminaton.                




===============================================================================
API Reference
=============

DDLAddIndex
-----------
This function adds one or more new indexes for the specified table
to that table's existing definitions. As this statement indicates,
you can add multiple index definitions in a single call to
DDLAddIndex.

        extern USHORT APIENTRY DDLAddIndex(USHORT iDictionaryHandle, 
                                           PSZ szTableName, 
                                           USHORT iIndexCount, 
                                           ADDINDEXMAP *pIndexList);

        iDictionaryHandle (input) -- The handle of the dictionary
           containing the table (or tables) for which an index
           is to be added (or for which indexes are to be added).
           The handle is obtained by using the DDLOpenDataDictionary 
           function.

        szTableName (input) -- A null-terminated string containing the 
           name of the table for which one or more indexes are to be 
           added.
  
        iIndexCount (input) -- An integer that specifies how many index 
        definitions are contained in the ADDINDEXMAP structure.

        pIndexList (input) -- A pointer to the structure that contains
           the key definitions that DDLAddIndex uses for adding the 
           index(es). The ADDINDEXMAP structure type is defined in the
           file DDLSVCS.H, and is discussed later in this document.

           You must specify index names in segment 0 of the index 
           definition. If the index is not defined as a named index, 
           set szIndexName to NULL. 

DDLCloseDataDictionary 
----------------------
This function closes a data dictionary.

        extern USHORT APIENTRY DDLCloseDataDictionary (USHORT iDictonaryHandle);

        iDictionaryHandle (input) -- The handle of the dictionary to 
           be  closed. The handle is obtained by using the 
           DDLOpenDataDictionary function.


DDLCreateDataDictionary 
-----------------------
This function creates a data dictionary; that is, it creates the
files FILE.DDF, FIELD.DDF, and INDEX.DDF in the specified directory.


        extern USHORT APIENTRY DDLCreateDataDictionary (PSZ szPathName,
                                                 PUSHORT *piDictionaryHandle,
                                                 PSZ hAltSeq); 

        szPathname (input) -- A null-terminated pathname that 
           identifies the directory containing the data dictionary; 
           that is, the directory that will contain the newly-created
           files FILE.DDF, FIELD.DDF, and INDEX.DDF.

        piDictionaryHandle (output) -- An integer pointer to the handle 
           of the newly-created dictionary. DDLCreateDataDictionary 
           returns this value, which must be used as an input parameter
           by most of the other functions in the DDL modules.

        hAltSeq (input) -- A handle that identifies the block of
           memory containing an alternate collating sequence (ACS). 
           You can create an ACS in one of two ways. 
           
           The first way is to write an application that creates an ACS 
           file. The internal format for the ACS file is described in the 
           Btrieve Programmer's Manual. Next, provide the ACS file's name 
           as one of the parameters to a user-defined resource statement 
           in your application's RC file. At this point, designate the 
           ACS as a resource handle in your C source code, and use the 
           Window's function FindResource to obtain the handle to the ACS 
           resource. 

           The other way to create an ACS is to define that ACS between
           the BEGIN and END keywords of an RCDATA statement in your 
           application's RC file. For the raw data, use the same ACS 
           format described for the ACS file in the Btrieve Programmer's 
           Manual. After completing the RCDATA statement, designate the 
           ACS as a resource handle in your C source code, and use
           the Window's function FindResource to obtain the handle to
           the ACS resource.

NOTE:      Your application does not need to lock the ACS resource 
           handle; the DDLSVCS module does this automatically. If
           your application does lock the handle, the DDLSVCS module
           will increment the lock count.

           If your application specifies a value of NULL for hAltSeq,
           DDLCreateDataDictionary uses the default ACS. 

After creating a data dictionary, you must create the tables that
describe the internal structure of your Btrieve data files. To create
tables, use the DDLCreateTable function.


DDLCreateTable
--------------
This function creates a table in the specified dictionary using the
table definition you supply in the associated TABLEPROPERTIES
structure.

Purpose of this API is to create Dictionary Definition for an already 
existing Btrieve file.  You can use this API to create a brand new Btrieve
file, however, you will not have the flexibility of providing the file
name for this brand new file.  DDLSVCS.DLL randomly will assign a file 
name to the Btrieve file and the file will get created in the current 
working directory.  If you attempt to create a Dictionary Definition 
for a Btrieve file that does not exisit, you must not specify a file 
name (szPathName).  If you do specify a file anme (szPathName), 
DDLCreateTable will fail with status 12.

  
extern USHORT APIENTRY DDLCreateTable  (USHORT iDictionaryHandle, 
                                        TABLEPROPERTIES *pTableProperties);


        iDictionaryHandle (input) -- The handle of the dictionary for 
           which you want to create a table. The handle is obtained by 
           using the DDLOpenDataDictionary function.

        pTableProperties (input) -- A pointer to the structure that 
           contains the property information DDLCreateTable uses for
           the table definition. The TABLEPROPERTIES structure type
           is defined in the file DDLSVCS.H, and is discussed later
           in this document.


DDLDropIndex
------------
This function drops (deletes) an index from an existing table in the 
specified dictionary. 

        extern USHORT APIENTRY DDLDropIndex (USHORT iDcitionary,
                                             PSZ szTableName,
                                             PSZ szIndexName,
                                             USHORT iIndex,
                                             PUSHORT *pIReorderIndexes);

        iDictionaryHandle (input) -- The handle of the dictionary
           containing the table from which an index is to be dropped.
           The handle is obtained by using the DDLOpenDataDictionary 
           function.

        szTableName (input) -- A null-terminated string containing the 
           name of the table from which an index is to be dropped.

        szIndexName (input) -- A null-terminated string containing the 
           name of the index to be dropped. To identify the index to
           be dropped, you either can supply the name of the index
           (in szIndexName), or the key number (in iIndex). If you
           specify any value for szIndexName other than NULL or a
           zero-length string (""), DDLDropIndex ignores any value 
           in iIndex.

        iIndex (input) -- An integer that specifies the key number of
           the index to be dropped. You can use either iIndex or
           szIndexName to identify the index to be dropped. If you
           use iIndex, (for example if the index to be dropped is not 
           a named index), you must set szIndexName to NULL or to a
           zero-length string ("").

        piReorderIndexes (input/output) -- This parameter, which applies
           only to Btrieve v6.0 and later, is an integer pointer to a 
           boolean flag. If you initialize this flag to FALSE, DDLDropIndex
           will not automatically renumber all keys with a key number
           greater than that of the deleted index. If you initialize
           the flag to TRUE, DDLDropIndex will renumber all keys in
           the same situation. 
 
NOTE:      If you specify that DDLDropIndex should automatically 
           renumber the keys of a Btrieve v5.x file, the function will 
           drop the the index as requested, but will return a value of 
           FALSE in piReorderIndexes, indicating that it could not
           renumber the keys. (You cannot automatically renumber the
           keys of a Btrieve v5.x file when you drop an index.) 

           If you specify that DDLDropIndex should automatically 
           renumber the keys of a Btrieve v6.x file, DDLDropIndex will 
           drop the index as requested and it will return a value of 
           TRUE in piReorderIndexes, indicating that the keys were 
           automatically renumbered. 

DDLDropTable
------------
This function drops (deletes) a table from the specified dictionary.


        extern USHORT APIENTRY DDLDropTable (USHORT iDictionaryHandle,
                                             PSZ szTableName);


        iDictionaryHandle (input) -- The handle of the dictionary
           containing the table to be deleted. The handle is obtained by 
           using the DDLOpenDataDictionary function.

        szTableName (input) -- A null-terminated string containing the 
           name of the table to be deleted.

DDLGetTableNames
----------------
This function gets a list of names of the tables defined in the
specified dictionary.


        extern USHORT APIENTRY DDLGetTableNames(USHORT iDictionaryHandle,
                                                PUSHORT *piTableCount,
                                                TABLELIST *pTableList); 

        iDictionaryHandle (input) -- The handle of the dictionary from 
           which you want to obtain the list of table names.

        piTableCount (output) -- A pointer to an integer that indicates
           how many TABLEPROPERTIES structures have been returned in the 
           memory block identified by the pTableList parameter. 

        pTableList (output) -- A pointer to the handle that identifies 
           to your application the block of memory containing the
           collection of TABLEPROPERTIES structures returned by this 
           function.

DDLGetTableProperties
---------------------
This function gets information about the specified table that is
located in the specified dictionary, and returns that information in
the designated TABLEPROPERTIES structure. 


        extern USHORT APIENTRY DDLGetTableProperties(USHORT iDictionaryHandle,
                                                     TABLEPROPERTIES *pTableProps); 

        iDictionaryHandle (input) -- The handle of the dictionary that 
           contains the table for which you are obtaining information. 

        pTableProps (input/output) -- A pointer to the structure that will
           receive the property information returned by this function. 
           However, before your application can call DDLGetTableProperties, 
           it must first specify which table in the dictionary about which 
           to obtain information. To do so, it must set the szTableName 
           element of pTableProps to a null-terminated string containing 
           the table's name.


DDLGetVersionInfo
-----------------
This function obtains information about the version of the DDLSVCS and 
Btrieve modules. The Btrieve module(s) may be either requester, client, 
or both. If both requester and client are active, DDLGetVersionInfo 
returns the requester's version first, and signifies it by a type of 'N'.


        extern USHORT APIENTRY DDLGetVersionInfo (DDL_VERSION_INFO *pVersionInfo);

        pVersionInfo (output) -- A pointer to the structure that receives 
           the version information returned by DDLGetVersionInfo. The 
           DDL_VERSION_INFO structure type is defined in the file 
           DDLSVCS.H, and is discussed later in this document.

DDLOpenDataDictionary 
--------------------- 
This function opens a data dictionary. The data dictionary is
comprised of the files FILE.DDF, FIELD.DDF, and INDEX.DDF which
reside in the specified directory.


        extern USHORT APIENTRY DDLOpenDataDictionary(PSZ szPathname, 
                                                     PUSHORT *piDictionaryHandle, 
                                                     PSZ szUserName,
                                                     PSZ szPassword);

        szPathname (input) -- A null-terminated pathname that 
           identifies the directory containing the data dictionary.
           
        piDictionaryHandle (output) -- An integer pointer to the 
           handle for the dictionary. DDLOpenDataDictionary returns 
           this value, which must be used as an input variable by 
           most of the other functions in the DDL modules.

        szUserName (input) -- A null-terminated string that
           identifies the name of the user. This value must be
           supplied only in cases where NetWare SQL has imposed
           security restrictions on the dictionary. In such a case, 
           DDLOpenDataDictionary requires a user name and password 
           before it will open a file (more specifically, before it 
           will open the .DDF files that comprise the data dictionary.

        szUserPassword (input) -- A null-terminated string that
           identifies the user's login password. This value must be
           supplied only in cases where NetWare SQL has imposed
           security restrictions on the dictionary. In such a case, 
           DDLOpenDataDictionary requires a user name and password 
           before it will open a file (more specifically, before it 
           will open the .DDF files that comprise the data dictionary.

NOTE:      If security is defined for the dictionary, a successful 
           opening of the data dictionary will include opening the 
           files USER.DDF and RIGHTS.DDF.

===============================================================================
Main Structures
===============

Following are descriptions of the main structures and their elements for
the DDLSVCS modules.

TABLEPROPERTIES
---------------
This structure defines the properties of a dictionary table. You must
supply a structure of type TABLEPROPERTIES whenever your application
calls either of the functions DDLGetTableProperties or DDLCreateTable.

When calling DDLCreateTable, your application must fill the
TABLEPROPERTIES structure with all necessary information to create the table.
This includes the lists of index and field descriptions.              

The DDLGetTableProperties returns information into the TABLEPROPERTIES 
structure.

Allocate the index and field lists as global memory objects. Do NOT
lock these objects either before or after calling DDLGetTableProperties 
or DDLCreateTable. This rule also applies when you supply an alternate 
collating sequence. Alternate collating sequences are passed to the
functions via a handle to a user-defined resource you must supply as
part of your RC file.

   BYTE   szTableName [21];       -- Contains the name of a table.
   BYTE   szPathName [65];        -- Contains the pathname of a Btrieve
                                     data file.
   USHORT iRecSize;               -- Contains the fixed record length of 
                                     the records in the Btrieve data file. 
   USHORT bVariable;              -- Indicates whether or not the Btrieve 
                                     data file contains variable-length 
                                     records.
   ULONG  dMaxRecSize;            -- Contains the maximum possible record 
                                     size; that is, the sum of the fixed 
                                     record length and the length of the 
                                     largest possible variable-length 
                                     record. 
   USHORT  bWriteAccess;           -- Indicates whether or not a user has 
                                     read/write access to a table.
   USHORT  iIndexCount;            -- Contains the count of items in the 
                                     index list. (See hIndexList.)     
   INDEXMAP hIndexList;            -- Reference the block of memory containing 
                                     the list of indexes for the table.
   USHORT   FieldCount;            -- Contains the count of items in the field 
                                     list. (See hFieldList.)
   FIELDMAP  hFieldList;           -- References the block of memory containing 
                                     the list of fields in the specified 
                                     table.
   PSZ    hAltSeq;                 -- References the block of memory containing 
                                     an alternate collating sequence.
                                                              

TABLELIST
---------
The TABLELIST structure is comprised of a series of null-terminated
strings, each of which contains the name of one of the tables defined 
in a dictionary. 

   BYTE   szName0[21];


FIELDMAP
--------
The FIELDMAP structure describes the properties of a specific field in 
a table.

   BYTE   szFieldName [21];       -- Contains the name of the field.
   USHORT iType;                  -- Identifies the type of data in
                                     the field.
   USHORT iOffset;                -- Indicates the offset within the
                                     record where this field begins.
   USHORT iLength;                -- Indicates the length of the field.
   USHORT iDecimal;               -- Indicates the number of decimal places
                                     required (for data types MONEY,
                                     DECIMAL, NUMERIC, and NUMERICSTS).
   BYTE   bAccessRights;          -- Indicates the user's access rights 
                                     on the field.


INDEXMAP
--------
The INDEXMAP structure describes the indexes associated with a table.

   USHORT iFieldIndex;            -- Contains the index into the
                                     FIELDMAP structure where
                                     information about the key begins.
   USHORT iIndex;                 -- Contains the Btrieve key number.
   USHORT iSegment;               -- Contains the key segment number.
   USHORT iFlags;                 -- Contains the collective value
                                     representing the attributes for
                                     this key (segmented, modifiable,
                                     duplicatable, and so on). See
                                     your Btrieve Programmer's Manual 
                                     for more information about key
                                     flags.


ADDINDEXMAP
-----------
The ADDINDEXMAP structure describes the indexes to be added to a table.

   BYTE   szFieldName[FIELDLEN];  -- Contains the name of the field 
                                     being added as an index.
   BYTE   szIndexName[FIELDLEN];  -- Contains the name of the index
                                     itself. If adding a named index, 
                                     the index name must be specified 
                                     in segment 0 of the key.
   USHORT   iIndex;                 -- Contains the Btrieve key number.
   USHORT   iSegment;               -- Contains the key segment number.
   USHORT   iFlags;                 -- Contains the collective value
                                     representing the attributes for
                                     this key (segmented, modifiable,
                                     duplicatable, and so on). See
                                     your Btrieve Programmer's Manual 
                                     for more information about key
                                     flags.


DDL_VERSION_INFO
----------------
The DDL_VERSION_INFO structure is used by the DDLGetVersionInfo function 
to hold the returned version information for both the DDLSVCS module and
the currently active Btrieve module(s).  

If only the Btrieve requester is active, it's version information is 
in the first group of versions, and its type is signified by 'N'.  

If only client-based Btrieve is active, it's version information is 
in the first group of versions, and its type is signified by 'P' for 
OS/2.

If both the requester and client-based Btrieve are active, the
requester's version information appears in the first group of Btrieve
version, and client-based Btrieve's version information appears in
the second group of Btrieve versions. 

   USHORT     iDDLVersion;     -- Contains the version number of the 
                                  DDLSVCS DDL.
   USHORT     iDDLRevision;    -- Contains the revision number of the
                                  DDLSVCS DDL.
   USHORT     iDDLType;        -- Identifies the type of DDLSVCS DDL
                                  currently loaded: 'W' indicates the
                                  OS/2 version.
   USHORT     iBVersion1;      -- Contains the version number of the
                                  currently loaded Btrieve. If more
                                  than one type of Btrieve is loaded,
                                  see iBVersion2.
   USHORT     iBRevision1;     -- Contains the revision number of the
                                  currently loaded Btrieve. If more
                                  than one type of Btrieve is loaded,
                                  see iBRevision2.
   USHORT     iBType1;         -- Identifies the type of Btrieve
                                  currently loaded. If more than one 
                                  type of Btrieve is loaded, see 
                                  iBType2.
                                     'N': Btrieve requester
                                     ' ': Client-based Btrieve (DOS)
                                     'W': Client-based Btrieve (OS/2)
   USHORT    iBVersion2;      -- Contains the version number of the
                                  second type of Btrieve currently
                                  loaded (if more than one type of 
                                  Btrieve is loaded).
   USHORT    iBRevision2;     -- Contains the revision number of the
                                  second type of Btrieve currently
                                  loaded (if more than one type of 
                                  Btrieve is loaded).
   USHORT    iBType2;         -- Identifies the type of the second
                                  type of Btrieve currently loaded (if 
                                  more than one type of Btrieve is 
                                  loaded). 
                                     'N': Btrieve requester
                                     ' ': Client-based Btrieve (DOS)
                                     'W': Client-based Btrieve (OS/2)


===============================================================================
Btrieve DDL Status Codes
========================

3001     DICTIONARY_OPEN     The dictionary has already been opened.
3002     NO_FIELDS_ERR       Returned from DDLCreateTable if no
                             fields have been specified in the 
                             TABLEPROPERTIES structure. The Btrieve
                             DDL module cannot create a table without
                             fields.
3003     ACCESS_DENIED       Access to the dictionary has been
                             denied. You must supply a valid username 
                             and password.
3004     FIELD_DEF_ERR       An invalid field definition was 
                             specified during a call to
                             DDLCreateTable.             

===============================================================================
List of Files
=============

Besides this file (README.DDL), the following files are included as
part of this product:

    DDLSVCS.H   
    DDLSVCS.DLL
    DDLSVCS.LIB 

===============================================================================
Customer Support
================

You can get support by posting your questions on Compuserve, BTRIEVE forum,
Section 5.  Support is also available by calling 1-800-BTRIEVE (Customers
outside of the U.S. and Canada can call 512-794-1719).  You can also send a
fax at 512-794-1778.

